﻿Imports System.Collections.Generic
Imports System.Linq
Imports System.Text
Imports System.Windows.Shapes
Imports System.Windows.Media
Imports System.Windows

Public Class Arc
    Inherits Shape
    Public Property StartAngle() As Double
        Get
            Return CDbl(GetValue(StartAngleProperty))
        End Get
        Set(ByVal value As Double)
            SetValue(StartAngleProperty, value)
        End Set
    End Property

    ' Using a DependencyProperty as the backing store for StartAngle.  This enables animation, styling, binding, etc...
    Public Shared ReadOnly StartAngleProperty As DependencyProperty = DependencyProperty.Register("StartAngle", GetType(Double), GetType(Arc), New UIPropertyMetadata(0.0, New PropertyChangedCallback(AddressOf UpdateArc)))

    Public Property EndAngle() As Double
        Get
            Return CDbl(GetValue(EndAngleProperty))
        End Get
        Set(ByVal value As Double)
            SetValue(EndAngleProperty, value)
        End Set
    End Property

    ' Using a DependencyProperty as the backing store for EndAngle.  This enables animation, styling, binding, etc...
    Public Shared ReadOnly EndAngleProperty As DependencyProperty = DependencyProperty.Register("EndAngle", GetType(Double), GetType(Arc), New UIPropertyMetadata(90.0, New PropertyChangedCallback(AddressOf UpdateArc)))


    Protected Shared Sub UpdateArc(ByVal d As DependencyObject, ByVal e As DependencyPropertyChangedEventArgs)
        Dim arc As Arc = TryCast(d, Arc)
        arc.InvalidateVisual()
    End Sub

    Protected Overrides ReadOnly Property DefiningGeometry() As Geometry
        Get
            Return GetArcGeometry()
        End Get
    End Property

    Protected Overrides Sub OnRender(ByVal drawingContext As System.Windows.Media.DrawingContext)
        drawingContext.DrawGeometry(Nothing, New Pen(Stroke, StrokeThickness), GetArcGeometry())
    End Sub

    Private Function GetArcGeometry() As Geometry
        Dim startPoint As Point = PointAtAngle(Math.Min(StartAngle, EndAngle))
        Dim endPoint As Point = PointAtAngle(Math.Max(StartAngle, EndAngle))

        Dim arcSize As New Size(Math.Max(0, (RenderSize.Width - StrokeThickness) / 2), Math.Max(0, (RenderSize.Height - StrokeThickness) / 2))
        Dim isLargeArc As Boolean = Math.Abs(EndAngle - StartAngle) > 180

        Dim geom As New StreamGeometry()
        Using context As StreamGeometryContext = geom.Open()
            context.BeginFigure(startPoint, False, False)
            context.ArcTo(endPoint, arcSize, 0, isLargeArc, SweepDirection.Counterclockwise, True, _
             False)
        End Using
        geom.Transform = New TranslateTransform(StrokeThickness / 2, StrokeThickness / 2)
        Return geom
    End Function

    Private Function PointAtAngle(ByVal angle As Double) As Point
        Dim radAngle As Double = angle * (Math.PI / 180)
        Dim xr As Double = (RenderSize.Width - StrokeThickness) / 2
        Dim yr As Double = (RenderSize.Height - StrokeThickness) / 2

        Dim x As Double = xr + xr * Math.Cos(radAngle)
        Dim y As Double = yr - yr * Math.Sin(radAngle)

        Return New Point(x, y)
    End Function
End Class
